@@ -128,6 +128,12 @@ pub enum RenderingError {
     /// A particular implementation-defined limit was exceeded.
     LimitExceeded(ImplementationLimit),
 
+    /// A non-invertible transform was generated.
+    ///
+    /// This should not be a fatal error; we should catch it and just not render
+    /// the problematic element.
+    InvalidTransform,
+
     /// Tried to reference an SVG element that does not exist.
     IdNotFound,
 
@@ -147,6 +153,12 @@ impl From<DefsLookupErrorKind> for RenderingError {
     }
 }
 
+impl From<InvalidTransform> for RenderingError {
+    fn from(_: InvalidTransform) -> RenderingError {
+        RenderingError::InvalidTransform
+    }
+}
+
 impl error::Error for RenderingError {}
 
 impl fmt::Display for RenderingError {
@@ -154,6 +166,7 @@ impl fmt::Display for RenderingError {
         match *self {
             RenderingError::Rendering(ref s) => write!(f, "rendering error: {s}"),
             RenderingError::LimitExceeded(ref l) => write!(f, "{l}"),
+            RenderingError::InvalidTransform => write!(f, "invalid transform"),
             RenderingError::IdNotFound => write!(f, "element id not found"),
             RenderingError::InvalidId(ref s) => write!(f, "invalid id: {s:?}"),
             RenderingError::OutOfMemory(ref s) => write!(f, "out of memory: {s}"),
@@ -167,6 +180,13 @@ impl From<cairo::Error> for RenderingError {
     }
 }
 
+/// Indicates that a transform is not invertible.
+///
+/// This generally represents an error from [`ValidTransform::try_from`], which is what we use
+/// to check affine transforms for validity.
+#[derive(Debug, PartialEq)]
+pub struct InvalidTransform;
+
 /// Errors from [`crate::document::AcquiredNodes`].
 pub enum AcquireError {
     /// An element with the specified id was not found.
@@ -224,9 +244,9 @@ impl fmt::Display for AcquireError {
 /// `set_atts()` methods as follows:
 ///
 /// ```
-/// # use librsvg::doctest_only::AttributeResultExt;
-/// # use librsvg::doctest_only::ValueErrorKind;
-/// # use librsvg::doctest_only::ElementError;
+/// # use rsvg::doctest_only::AttributeResultExt;
+/// # use rsvg::doctest_only::ValueErrorKind;
+/// # use rsvg::doctest_only::ElementError;
 /// # use markup5ever::{QualName, Prefix, Namespace, LocalName};
 /// # type FooError = ValueErrorKind;
 /// fn parse_foo(value: &str) -> Result<(), FooError>
@@ -313,6 +333,12 @@ pub enum AllowedUrlError {
     /// or in one directory below the base file.
     NotSiblingOrChildOfBaseFile,
 
+    /// Loaded file:// URLs cannot have a query part, e.g. `file:///foo?blah`
+    NoQueriesAllowed,
+
+    /// URLs may not have fragment identifiers at this stage
+    NoFragmentIdentifierAllowed,
+
     /// Error when obtaining the file path or the base file path
     InvalidPath,
 
@@ -325,17 +351,18 @@ pub enum AllowedUrlError {
 
 impl fmt::Display for AllowedUrlError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        use AllowedUrlError::*;
         match *self {
-            AllowedUrlError::UrlParseError(e) => write!(f, "URL parse error: {e}"),
-            AllowedUrlError::BaseRequired => write!(f, "base required"),
-            AllowedUrlError::DifferentUriSchemes => write!(f, "different URI schemes"),
-            AllowedUrlError::DisallowedScheme => write!(f, "disallowed scheme"),
-            AllowedUrlError::NotSiblingOrChildOfBaseFile => {
-                write!(f, "not sibling or child of base file")
-            }
-            AllowedUrlError::InvalidPath => write!(f, "invalid path"),
-            AllowedUrlError::BaseIsRoot => write!(f, "base is root"),
-            AllowedUrlError::CanonicalizationError => write!(f, "canonicalization error"),
+            UrlParseError(e) => write!(f, "URL parse error: {e}"),
+            BaseRequired => write!(f, "base required"),
+            DifferentUriSchemes => write!(f, "different URI schemes"),
+            DisallowedScheme => write!(f, "disallowed scheme"),
+            NotSiblingOrChildOfBaseFile => write!(f, "not sibling or child of base file"),
+            NoQueriesAllowed => write!(f, "no queries allowed"),
+            NoFragmentIdentifierAllowed => write!(f, "no fragment identifier allowed"),
+            InvalidPath => write!(f, "invalid path"),
+            BaseIsRoot => write!(f, "base is root"),
+            CanonicalizationError => write!(f, "canonicalization error"),
         }
     }
 }
